import { Body, Controller, Logger } from "@nestjs/common";

import { ProducerService } from "@ocm-engine/nats";
import { MessagePattern, RpcException } from "@nestjs/microservices";
import {
  CloudEventDto,
  ConnectionEvent,
  CreateInvitationResponseDto,
  GatewayAcceptedResponseDto,
  GetConnectionRequestDto,
  makeEvent,
} from "@ocm-engine/dtos";

@Controller()
export class AppController {
  private readonly logger: Logger = new Logger(AppController.name);

  constructor(private readonly producerService: ProducerService) {}

  @MessagePattern("connections")
  async create(
    @Body()
    payload: {
      data: null | CreateInvitationResponseDto | GetConnectionRequestDto;
      type: ConnectionEvent;
      source: string;
    },
  ): Promise<GatewayAcceptedResponseDto> {
    this.logger.debug(JSON.stringify(payload, null, 2));

    try {
      const event = makeEvent(payload);
      this.logger.debug(JSON.stringify(event, null, 2));

      await this.producerService.publish<typeof payload.data>(
        payload.type,
        event as CloudEventDto<typeof payload.data>,
      );

      const response = new GatewayAcceptedResponseDto();
      response.id = event.id;

      return response;
    } catch (e) {
      this.logger.debug(JSON.stringify(e, null, 2));
      if (e instanceof Error) {
        throw new RpcException(e.message);
      }

      throw new RpcException("Internal server error");
    }
  }
}
