import {
  Injectable,
  Logger,
  OnModuleDestroy,
  OnModuleInit,
} from "@nestjs/common";
import { AgentService } from "../askar/agent.service";
import { ConsumerService } from "@ocm-engine/nats";
import { ConfigService } from "@nestjs/config";
import { IConfAgent } from "@ocm-engine/config";
import { GatewayClient } from "@ocm-engine/clients";
import { EventHandlerService } from "./event.handler.service";

@Injectable()
export class AgentConsumerService implements OnModuleInit, OnModuleDestroy {
  private readonly logger = new Logger(AgentConsumerService.name);
  constructor(
    private readonly agentService: AgentService,
    private readonly consumerService: ConsumerService,
    private readonly configService: ConfigService,
    private readonly gatewayClient: GatewayClient,
    private readonly eventHandlerService: EventHandlerService,
  ) {}

  async onModuleInit(): Promise<void> {
    const config = this.configService.get<IConfAgent>("agent");

    if (config?.agentIsRest) {
      this.logger.log(
        "Agent is configured as rest, there is no need for consumer!",
      );
      return;
    }

    await this.consumerService.subscribe(async (event) => {
      this.logger.debug(JSON.stringify(event, null, 2));
      let ev;

      try {
        ev = await this.eventHandlerService.handle<typeof event.data>(event);
        this.logger.debug(JSON.stringify(ev));
      } catch (e) {
        this.logger.debug({ e: JSON.stringify(e, null, 2) });
        event.data = e;
        ev = event;
      }

      this.gatewayClient.sendPayload(ev);

      return;
    });
  }

  async onModuleDestroy(): Promise<void> {
    this.logger.log("disconnecting from broker");
    await this.consumerService.disconnect();
  }
}
