import { Body, Controller, Get, Param, Post, UseFilters } from "@nestjs/common";
import { AgentService } from "../askar/agent.service";
import {
  CreateCredentialDefinitionRequsetDto,
  CreateInvitationResponseDto,
  IssueCredentialRequestDto,
  IssueProofRequestDto,
  CreateSchemaRequestDto,
  MakeBasicMessageRequestDto,
} from "@ocm-engine/dtos";
import { AllExceptionsHandler } from "./exception.handler";

@UseFilters(AllExceptionsHandler)
@Controller("v1")
export class RestController {
  constructor(private readonly agentService: AgentService) {}

  @Post("/invitation")
  createInvitation() {
    return this.agentService.createInvitation();
  }

  @Post("/invitation/accept")
  acceptInvitation(@Body() createInvitationDto: CreateInvitationResponseDto) {
    return this.agentService.acceptInvitation(
      createInvitationDto.invitationUrl,
    );
  }

  @Get("connections")
  connections() {
    return this.agentService.fetchConnections();
  }

  @Post("schema")
  createSchema(@Body() schemaDto: CreateSchemaRequestDto) {
    return this.agentService.createSchema(schemaDto);
  }

  @Post("credential/definition")
  createCredentialDefinition(
    @Body() credentialDefinitionDto: CreateCredentialDefinitionRequsetDto,
  ) {
    return this.agentService.createCredentialDefinition(
      credentialDefinitionDto,
    );
  }

  @Post("credential/issue")
  issueCredential(@Body() issueCredentialDto: IssueCredentialRequestDto) {
    return this.agentService.issueCredential(issueCredentialDto);
  }

  @Get("/credentials")
  credentials() {
    return this.agentService.credentials();
  }

  @Get("/credential/offers")
  getCredentialOffers() {
    return this.agentService.credentialByStatedOfferReceived();
  }

  @Post("/credential/:credential_record_id/accept")
  acceptCredential(@Param("credential_record_id") credentialRecordId: string) {
    return this.agentService.acceptCredential(credentialRecordId);
  }

  @Get("/credential/proof")
  proofs() {
    return this.agentService.proofs();
  }

  @Post("credential/proof/issue")
  issueProof(@Body() issueProofDto: IssueProofRequestDto) {
    return this.agentService.issueProof(issueProofDto);
  }

  @Post(`/credential/proof/:proof_record_id/accept`)
  acceptProof(@Param("proof_record_id") proofRecordId: string) {
    return this.agentService.acceptProof(proofRecordId);
  }

  @Post("/resolve")
  resolve(@Body("did") did: string) {
    return this.agentService.resolve(did);
  }

  @Post("/messages")
  sendMeesage(@Body() message: MakeBasicMessageRequestDto) {
    return this.agentService.sendMessage(message);
  }
}
