import { Injectable, Logger } from "@nestjs/common";
import {
  ClientProxy,
  ClientProxyFactory,
  Transport,
} from "@nestjs/microservices";
import { ConfigService } from "@nestjs/config";
import { CloudEventDto } from "@ocm-engine/dtos";
import { IGateway } from "@ocm-engine/config";

@Injectable()
export class GatewayClient {
  private client: ClientProxy;
  private gatewayConf: IGateway;
  private readonly logger: Logger = new Logger(GatewayClient.name);

  constructor(configService: ConfigService) {
    this.gatewayConf = configService.get<IGateway>("gateway")!;

    this.client = ClientProxyFactory.create({
      transport: Transport.TCP,
      options: {
        host: this.gatewayConf.host,
        port: this.gatewayConf.tcpPort,
      },
    });
  }

  sendPayload<T>(payload: CloudEventDto<T>) {
    this.logger.debug(
      `sending payload to gateway ${JSON.stringify(payload, null, 2)}`,
    );

    return this.client.emit(this.gatewayConf.messagePattern, payload);
  }
}
