import {
  ArrayMinSize,
  IsArray,
  IsNotEmpty,
  IsString,
  ValidateNested,
} from "class-validator";
import { Type } from "class-transformer";

export class IssueCredentialAttributes {
  @IsString()
  name: string;

  @IsString()
  value: string;
}

export class IssueCredentialRequestDto {
  //@example 6464b521-005a-4379-91e0-a3692b31cafd
  @IsString()
  @IsNotEmpty()
  connectionId: string;

  //@example did:indy:LEDGER:5Pf67NsEtnU42GJGETT6Xe/anoncreds/v0/CLAIM_DEF/855916/test mest cred def
  @IsString()
  @IsNotEmpty()
  credentialDefinitionId: string;

  @IsArray()
  @ArrayMinSize(1)
  @ValidateNested({ each: true })
  @Type(() => IssueCredentialAttributes)
  attributes: Array<IssueCredentialAttributes>;
}
