import {
  IsArray,
  IsDateString,
  IsNotEmpty,
  IsString,
  ValidateNested,
} from "class-validator";
import { Type } from "class-transformer";
import { IssueCredentialAttributes } from "../requests/issue.credential.request.dto";

export class Credential {
  @IsNotEmpty()
  @IsString()
  credentialRecordType: string;

  @IsNotEmpty()
  @IsString()
  credentialRecordId: string;
}

export class IssueCredentialResponseDto {
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Credential)
  credentials?: Array<Credential>;

  @IsNotEmpty()
  @IsString()
  credentialId: string;

  @IsString()
  connectionId?: string;

  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => IssueCredentialAttributes)
  attributes?: Array<IssueCredentialAttributes>;

  @IsNotEmpty()
  @IsDateString()
  createdAt: Date;
}
